@@ -1,43 +1,26 @@
+import time
 import logging
-import os
-import queue
-import random
-import shutil
-import subprocess
-import threading
 import tkinter as tk
-import tkinter.messagebox as tkMessageBox
-import webbrowser
-from pathlib import Path
 from tkinter import PhotoImage, ttk
-from tkinter.scrolledtext import ScrolledText
+from multiprocessing import Queue
 
-import requests
-from PIL import Image, ImageTk
-
-from modlunky2.assets.assets import AssetStore
-from modlunky2.assets.constants import (
-    EXTRACTED_DIR,
-    FILEPATH_DIRS,
-    OVERRIDES_DIR,
-    PACKS_DIR,
-)
-from modlunky2.assets.exc import MissingAsset
-from modlunky2.assets.patcher import Patcher
 from modlunky2.constants import BASE_DIR, IS_EXE
-from modlunky2.version import current_version, latest_version
 from modlunky2.updater import self_update
-from modlunky2.ui.extract import ExtractTab
-from modlunky2.ui.levels import LevelsTab
-from modlunky2.ui.pack import PackTab
-from modlunky2.ui.config import ConfigTab
-from modlunky2.ui.widgets import ConsoleWindow
+from modlunky2.version import current_version, latest_version
+
+from .tasks import TaskManager, PING_INTERVAL
+from .config import ConfigTab
+from .extract import ExtractTab
+from .levels import LevelsTab
+from .pack import PackTab
+from .widgets import ConsoleWindow
+from .logs import QueueHandler, register_queue_handler
 
 logger = logging.getLogger("modlunky2")
 
 
-MIN_WIDTH = 1024
-MIN_HEIGHT = 800
+MIN_WIDTH = 1280
+MIN_HEIGHT = 900
 
 
 class ModlunkyUI:
@@ -45,21 +28,25 @@ class ModlunkyUI:
         self.config = config
         self.data_dir = data_dir
 
+        self.current_version = current_version()
+        self.latest_version = latest_version()
+
         if IS_EXE:
-            self.current_version = current_version()
-            self.latest_version = latest_version()
             if self.latest_version is None or self.current_version is None:
                 self.needs_update = False
             else:
                 self.needs_update = self.current_version < self.latest_version
         else:
-            self.current_version = None
-            self.latest_version = None
             self.needs_update = False
 
         self._shutdown_handlers = []
         self._shutting_down = False
 
+        self.log_queue = Queue()
+        self.queue_handler = QueueHandler(self.log_queue)
+        register_queue_handler(self.queue_handler)
+        self.task_manager = TaskManager(self.log_queue)
+
         self.root = tk.Tk(className="Modlunky2")  # Equilux Black
         self.root.title("Modlunky 2")
         self.root.geometry(f"{MIN_WIDTH}x{MIN_HEIGHT}")
@@ -74,14 +61,18 @@ class ModlunkyUI:
 
         if self.needs_update:
             update_frame = ttk.LabelFrame(self.root, text="Modlunky2 Update Available")
-            update_frame.grid(row=0, column=0, pady=5, padx=5, sticky="nswe")
+            update_frame.grid(row=0, column=0, sticky="nswe")
             update_frame.columnconfigure(0, weight=1)
             update_frame.rowconfigure(0, weight=1)
 
             update_button = tk.Button(
-                update_frame, text="Update Now!", command=self.update, bg="#bfbfbf", font="sans 12 bold"
+                update_frame,
+                text="Update Now!",
+                command=self.update,
+                bg="#bfbfbf",
+                font="sans 12 bold",
             )
-            update_button.grid(column=0, row=0, padx=5, pady=5, sticky="nswe")
+            update_button.grid(column=0, row=0, sticky="nswe")
 
         # Handle shutting down cleanly
         self.root.protocol("WM_DELETE_WINDOW", self.quit)
@@ -94,6 +85,7 @@ class ModlunkyUI:
             PackTab(
                 tab_control=self.tab_control,
                 config=config,
+                task_manager=self.task_manager,
             ),
         )
         self.register_tab(
@@ -101,10 +93,11 @@ class ModlunkyUI:
             ExtractTab(
                 tab_control=self.tab_control,
                 config=config,
+                task_manager=self.task_manager,
             ),
         )
         self.register_tab(
-            "Levels",
+            "Level Editor",
             LevelsTab(
                 tab_control=self.tab_control,
                 config=config,
@@ -122,8 +115,49 @@ class ModlunkyUI:
         self.tab_control.bind("<<NotebookTabChanged>>", self.on_tab_change)
         self.tab_control.grid(column=0, row=1, padx=2, pady=(4, 0), sticky="nsew")
 
-        self.console = ConsoleWindow(self.root)
-        self.console.grid(column=0, row=2, padx=2, pady=2, sticky="ew")
+        self.console_frame = ttk.LabelFrame(self.root, text="Console")
+        self.console_frame.grid(row=2, column=0, padx=5, pady=(5, 0), sticky="nswe")
+        self.console_frame.columnconfigure(0, weight=1)
+        self.console_frame.rowconfigure(0, weight=1)
+
+        self.console = ConsoleWindow(self.queue_handler, self.console_frame)
+        self.console.grid(column=0, row=0, padx=2, pady=2, sticky="ew")
+
+        self.version_label = tk.Label(
+            self.root, text=f"v{self.current_version}", font="Helvitica 9 italic"
+        )
+        self.version_label.grid(column=0, row=3, padx=5, sticky="e")
+
+        self.task_manager.start_process()
+        self.last_ping = time.time()
+        self.root.after(100, self.after_cb)
+
+    def after_cb(self):
+        if not self.task_manager.is_alive():
+            # Worker process went away but we're shutting down so just return
+            if self._shutting_down:
+                return
+
+            # Worker process went away unexpectedly... Restart it.
+            logger.critical("Worker process went away... Restarting it.")
+            self.task_manager.start_process()
+            self.root.after(100, self.after_cb)
+            return
+
+        # Send regular pings so the worker process knows
+        # we're still alive.
+        now = time.time()
+        if now - self.last_ping > PING_INTERVAL:
+            self.last_ping = now
+            self.task_manager.ping()
+
+        while True:
+            msg = self.task_manager.receive_message()
+            if msg is None:
+                self.root.after(100, self.after_cb)
+                return
+
+            self.task_manager.dispatch(msg)
 
     def update(self):
         try:
@@ -149,6 +183,7 @@ class ModlunkyUI:
         for handler in self._shutdown_handlers:
             handler()
 
+        self.task_manager.quit()
         self.root.quit()
         self.root.destroy()
 
